@@ -109,7 +109,7 @@ compute_powa_frequency(void)
 	int local_frequency = powa_frequency;
 
 	/*
-	 * If PoWA is deactivated, artitrarily set a sleep time of one hour to save
+	 * If PoWA is deactivated, arbitrarily set a sleep time of one hour to save
 	 * resources.  The actual sleep time is not problematic since the
 	 * reactivation can only be done with a sighup, which will set the latch
 	 * used for the sleep.
@@ -159,18 +159,38 @@ compute_next_wakeup(void)
 	return INSTR_TIME_GET_MICROSEC(time_to_wait);
 }
 
+/*
+ * As of powa 4, this extension can be with a remote snapshot daemon instead of
+ * the dedicated background worker.  In order to allow this daemon to use the
+ * statistic functions powa provide, we now allow to load the extension from
+ * outside shared_preload_libraries.
+ */
 void
 _PG_init(void)
 {
 	BackgroundWorker worker;
 
-	if (!process_shared_preload_libraries_in_progress)
-	{
-		elog(ERROR, "This module can only be loaded via shared_preload_libraries");
-		return;
-	}
+	/*
+	 * This setting can be used for datasource function, so we always define
+	 * this GUC
+	 */
+	DefineCustomStringVariable("powa.ignored_users",
+							   "Defines a coma-separated list of users to ignore when taking activity snapshot",
+							   NULL,
+							   &powa_ignored_users,
+							   NULL, PGC_SIGHUP, 0, NULL, NULL, NULL);
 
+	DefineCustomBoolVariable("powa.debug",
+							   "Provide logs to help troubleshooting issues",
+							   NULL,
+							   &powa_debug,
+							   false, PGC_USERSET, 0, NULL, NULL, NULL);
 
+	/*
+	 * The rest of the GUCs are not required when the bgworker isn't active,
+	 * but it can be useful when manually calling powa_take_snapshot(), and
+	 * defining them doesn't hurt anyway
+	 */
 	DefineCustomIntVariable("powa.frequency",
 						 "Defines the frequency in seconds of the snapshots",
 							NULL,
@@ -198,29 +218,25 @@ _PG_init(void)
 							INT_MAX / SECS_PER_MINUTE,
 							PGC_SUSET, GUC_UNIT_MIN, NULL, NULL, NULL);
 
+	EmitWarningsOnPlaceholders("powa");
+
+	/*
+	 * Following code is only needed for the bgworker, so only used when powa
+	 * is loaded in shared_preload_libraries.
+	 */
+	if (!process_shared_preload_libraries_in_progress)
+		return;
+
 	DefineCustomStringVariable("powa.database",
 						   "Defines the database of the workload repository",
 							   NULL,
 							   &powa_database,
 							   "powa", PGC_POSTMASTER, 0, NULL, NULL, NULL);
 
-	DefineCustomStringVariable("powa.ignored_users",
-							   "Defines a coma-separated list of users to ignore when taking activity snapshot",
-							   NULL,
-							   &powa_ignored_users,
-							   NULL, PGC_SIGHUP, 0, NULL, NULL, NULL);
-
-	DefineCustomBoolVariable("powa.debug",
-							   "Provide logs to help troubleshooting issues",
-							   NULL,
-							   &powa_debug,
-							   false, PGC_USERSET, 0, NULL, NULL, NULL);
-
-	EmitWarningsOnPlaceholders("powa");
-
 	/*
 	 * Register the worker processes
 	 */
+	memset(&worker, 0, sizeof(worker));
 	worker.bgw_flags =
 		BGWORKER_SHMEM_ACCESS | BGWORKER_BACKEND_DATABASE_CONNECTION;
 	worker.bgw_start_time = BgWorkerStart_RecoveryFinished;		/* Must write to the
@@ -244,8 +260,8 @@ _PG_init(void)
 void
 powa_main(Datum main_arg)
 {
-	char	   *query_snapshot = "SELECT powa_take_snapshot()";
-	static char *query_appname = "SET application_name = 'PoWA collector'";
+	char	   *query_snapshot = "SELECT public.powa_take_snapshot()";
+	static char *query_appname = "SET application_name = 'PoWA - collector'";
 	int64		us_to_wait; /* Should be uint64 per postgresql's spec, but we
 							   may have negative result, in our tests */
 
@@ -259,6 +275,36 @@ powa_main(Datum main_arg)
 
 	BackgroundWorkerUnblockSignals();
 
+	if (powa_frequency == -1)
+	{
+		elog(LOG, "PoWA is deactivated");
+		pgstat_report_activity(STATE_IDLE, NULL);
+	}
+
+	/*
+	 * First check if local snapshot is disabled, and sleep at that point if
+	 * that's the case.  This avoids spurious logging if powa is in
+	 * shared_preload_libraries but the target database hasn't been created.
+	 */
+	while (powa_frequency == -1)
+	{
+		/* Check if a SIGHUP has been received */
+		powa_process_sighup();
+
+		if (powa_frequency != -1)
+			break;
+
+		/* sleep */
+		WaitLatch(&MyProc->procLatch,
+				  WL_LATCH_SET | WL_TIMEOUT | WL_POSTMASTER_DEATH,
+				  3600000
+#if PG_VERSION_NUM >= 100000
+				  ,PG_WAIT_EXTENSION
+#endif
+				  );
+		ResetLatch(&MyProc->procLatch);
+	}
+
 	/* Define the snapshot reference time when the bgworker start */
 	INSTR_TIME_SET_CURRENT(last_start);
 
@@ -271,7 +317,11 @@ powa_main(Datum main_arg)
 
 	elog(LOG, "POWA connected to database %s", quote_identifier(powa_database));
 
-	set_ps_display("init", false);
+	set_ps_display("init"
+#if PG_VERSION_NUM < 130000
+			, false
+#endif
+			);
 	StartTransactionCommand();
 	SetCurrentStatementStartTimestamp();
 	SPI_connect();
@@ -282,7 +332,11 @@ powa_main(Datum main_arg)
 	PopActiveSnapshot();
 	CommitTransactionCommand();
 	pgstat_report_activity(STATE_IDLE, NULL);
-	set_ps_display("idle", false);
+	set_ps_display("idle"
+#if PG_VERSION_NUM < 130000
+			, false
+#endif
+			);
 
 	/*------------------
 	 * Main loop of POWA
@@ -297,7 +351,11 @@ powa_main(Datum main_arg)
 
 		if (powa_frequency != -1)
 		{
-			set_ps_display("snapshot", false);
+			set_ps_display("snapshot"
+#if PG_VERSION_NUM < 130000
+			, false
+#endif
+			);
 			SetCurrentStatementStartTimestamp();
 			StartTransactionCommand();
 			SPI_connect();
@@ -311,7 +369,11 @@ powa_main(Datum main_arg)
 			CommitTransactionCommand();
 			pgstat_report_stat(false);
 			pgstat_report_activity(STATE_IDLE, NULL);
-			set_ps_display("idle", false);
+			set_ps_display("idle"
+#if PG_VERSION_NUM < 130000
+			, false
+#endif
+			);
 		}
 
 		/* sleep loop */
